@@ -3,6 +3,7 @@ import time
 import warnings
 from base64 import b32encode
 from binascii import unhexlify
+from inspect import signature
 from uuid import uuid4
 
 import django_otp
@@ -20,6 +21,7 @@ from django.shortcuts import redirect, resolve_url
 from django.urls import reverse
 from django.utils.decorators import method_decorator
 from django.utils.functional import cached_property
+from django.utils.http import url_has_allowed_host_and_scheme
 from django.utils.module_loading import import_string
 from django.utils.translation import gettext as _
 from django.views.decorators.cache import never_cache
@@ -33,11 +35,10 @@ from django_otp.plugins.otp_static.models import StaticDevice, StaticToken
 from django_otp.util import random_hex
 
 from two_factor import signals
-from two_factor.plugins.phonenumber.utils import (
-    backup_phones, get_available_phone_methods,
-)
+from two_factor.plugins.phonenumber.utils import get_available_phone_methods
 from two_factor.plugins.registry import registry
 from two_factor.utils import totp_digits
+from two_factor.views.mixins import OTPRequiredMixin
 
 from ..forms import (
     AuthenticationTokenForm, BackupTokenForm, DeviceValidationForm, MethodForm,
@@ -49,20 +50,12 @@ from .utils import (
     get_remember_device_cookie, validate_remember_device_cookie,
 )
 
-try:
-    from django.utils.http import url_has_allowed_host_and_scheme
-except ImportError:  # django<3.0
-    from django.utils.http import (
-        is_safe_url as url_has_allowed_host_and_scheme,
-    )
-
 try:
     from django.contrib.auth.views import RedirectURLMixin
 except ImportError:  # django<4.1
     from django.contrib.auth.views import (
         SuccessURLAllowedHostsMixin as RedirectURLMixin,
     )
-
 logger = logging.getLogger(__name__)
 
 REMEMBER_COOKIE_PREFIX = getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_PREFIX', 'remember-cookie_')
@@ -141,6 +134,7 @@ class LoginView(RedirectURLMixin, IdempotentSessionWizardView):
 
         # Generating a challenge doesn't require to validate the form.
         if 'challenge_device' in self.request.POST:
+            self.storage.data['challenge_device'] = self.request.POST['challenge_device']
             return self.render_goto_step('token')
 
         response = super().post(*args, **kwargs)
@@ -181,6 +175,13 @@ class LoginView(RedirectURLMixin, IdempotentSessionWizardView):
                                     httponly=getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_HTTPONLY', True),
                                     samesite=getattr(settings, 'TWO_FACTOR_REMEMBER_COOKIE_SAMESITE', 'Lax'),
                                     )
+            return response
+
+        # If the user does not have a device.
+        elif OTPRequiredMixin.is_otp_view(self.request.GET.get('next')):
+            if self.request.GET.get('next'):
+                self.request.session['next'] = self.get_success_url()
+            return redirect('two_factor:setup')
 
         return response
 
@@ -190,7 +191,7 @@ class LoginView(RedirectURLMixin, IdempotentSessionWizardView):
         url = self.get_redirect_url()
         return url or resolve_url(settings.LOGIN_REDIRECT_URL)
 
-    # Copied from django.conrib.auth.views.LoginView (Branch: stable/1.11.x)
+    # Copied from django.contrib.auth.views.LoginView (Branch: stable/1.11.x)
     # https://github.com/django/django/blob/58df8aa40fe88f753ba79e091a52f236246260b3/django/contrib/auth/views.py#L67
     def get_redirect_url(self):
         """Return the user-originating redirect URL if it's safe."""
@@ -206,19 +207,20 @@ class LoginView(RedirectURLMixin, IdempotentSessionWizardView):
         return redirect_to if url_is_safe else ''
 
     def get_form_kwargs(self, step=None):
-        """
-        AuthenticationTokenForm requires the user kwarg.
-        """
-        if step == 'auth':
-            return {
-                'request': self.request
-            }
-        if step in ('token', 'backup'):
-            return {
-                'user': self.get_user(),
-                'initial_device': self.get_device(step),
-            }
-        return {}
+        if step is None:
+            return {}
+
+        form_class = self.get_form_list()[step]
+        form_params = signature(form_class).parameters
+
+        kwargs = {}
+        if 'user' in form_params:
+            kwargs['user'] = self.get_user()
+        if 'initial_device' in form_params:
+            kwargs['initial_device'] = self.get_device(step)
+        if 'request' in form_params:
+            kwargs['request'] = self.request
+        return kwargs
 
     def get_done_form_list(self):
         """
@@ -277,21 +279,44 @@ class LoginView(RedirectURLMixin, IdempotentSessionWizardView):
         Returns the OTP device selected by the user, or his default device.
         """
         if not self.device_cache:
-            challenge_device_id = self.request.POST.get('challenge_device', None)
+            challenge_device_id = (
+                self.request.POST.get('challenge_device')
+                or self.storage.data.get('challenge_device')
+            )
             if challenge_device_id:
-                for device in backup_phones(self.get_user()):
+                for device in self.get_devices():
                     if device.persistent_id == challenge_device_id:
                         self.device_cache = device
                         break
+
             if step == 'backup':
                 try:
                     self.device_cache = self.get_user().staticdevice_set.get(name='backup')
                 except StaticDevice.DoesNotExist:
                     pass
+
             if not self.device_cache:
                 self.device_cache = default_device(self.get_user())
+
         return self.device_cache
 
+    def get_devices(self):
+        user = self.get_user()
+
+        devices = []
+        for method in registry.get_methods():
+            devices += list(method.get_devices(user))
+        return devices
+
+    def get_other_devices(self, main_device):
+        user = self.get_user()
+
+        other_devices = []
+        for method in registry.get_methods():
+            other_devices += list(method.get_other_authentication_devices(user, main_device))
+
+        return other_devices
+
     def render(self, form=None, **kwargs):
         """
         If the user selected a device, ask the device to generate a challenge;
@@ -318,10 +343,10 @@ class LoginView(RedirectURLMixin, IdempotentSessionWizardView):
         """
         context = super().get_context_data(form, **kwargs)
         if self.steps.current == 'token':
-            context['device'] = self.get_device()
-            context['other_devices'] = [
-                phone for phone in backup_phones(self.get_user())
-                if phone != self.get_device()]
+            device = self.get_device()
+            context['device'] = device
+            context['other_devices'] = self.get_other_devices(device)
+
             try:
                 context['backup_tokens'] = self.get_user().staticdevice_set\
                     .get(name='backup').token_set.count()
@@ -375,7 +400,7 @@ class LoginView(RedirectURLMixin, IdempotentSessionWizardView):
             response.delete_cookie(cookie)
         return response
 
-    # Copied from django.conrib.auth.views.LoginView  (Branch: stable/1.11.x)
+    # Copied from django.contrib.auth.views.LoginView  (Branch: stable/1.11.x)
     # https://github.com/django/django/blob/58df8aa40fe88f753ba79e091a52f236246260b3/django/contrib/auth/views.py#L49
     @method_decorator(sensitive_post_parameters())
     @method_decorator(csrf_protect)
@@ -394,7 +419,7 @@ class LoginView(RedirectURLMixin, IdempotentSessionWizardView):
 
 @class_view_decorator(never_cache)
 @class_view_decorator(login_required)
-class SetupView(IdempotentSessionWizardView):
+class SetupView(RedirectURLMixin, IdempotentSessionWizardView):
     """
     View for handling OTP setup using a wizard.
 
@@ -417,6 +442,27 @@ class SetupView(IdempotentSessionWizardView):
         # Other forms are dynamically added in get_form_list()
     )
 
+    # Copied from django.contrib.auth.views.LoginView (Branch: stable/1.11.x)
+    # https://github.com/django/django/blob/58df8aa40fe88f753ba79e091a52f236246260b3/django/contrib/auth/views.py#L63
+    def get_success_url(self):
+        url = self.get_redirect_url()
+        return url or reverse(self.success_url)
+
+    # Copied from django.contrib.auth.views.LoginView (Branch: stable/1.11.x)
+    # https://github.com/django/django/blob/58df8aa40fe88f753ba79e091a52f236246260b3/django/contrib/auth/views.py#L67
+    def get_redirect_url(self):
+        """Return the user-originating redirect URL if it's safe."""
+        redirect_to = self.request.POST.get(
+            REDIRECT_FIELD_NAME,
+            self.request.GET.get(REDIRECT_FIELD_NAME, '')
+        )
+        url_is_safe = url_has_allowed_host_and_scheme(
+            url=redirect_to,
+            allowed_hosts=self.get_success_url_allowed_hosts(),
+            require_https=self.request.is_secure(),
+        )
+        return redirect_to if url_is_safe else ''
+
     def get_method(self):
         method_data = self.storage.validated_step_data.get('method', {})
         method_key = method_data.get('method', None)
@@ -427,7 +473,7 @@ class SetupView(IdempotentSessionWizardView):
         Start the setup wizard. Redirect if already enabled.
         """
         if default_device(self.request.user):
-            return redirect(self.success_url)
+            return redirect(self.get_success_url())
         return super().get(request, *args, **kwargs)
 
     def get_form(self, step=None, **kwargs):
@@ -442,7 +488,7 @@ class SetupView(IdempotentSessionWizardView):
         """
         form_list = super().get_form_list()
 
-        available_methods = registry.get_methods()
+        available_methods = self.get_available_methods()
         if len(available_methods) == 1:
             form_list.pop('method', None)
             method_key = available_methods[0].code
@@ -457,6 +503,9 @@ class SetupView(IdempotentSessionWizardView):
             form_list['validation'] = DeviceValidationForm
         return form_list
 
+    def get_available_methods(self):
+        return registry.get_methods()
+
     def render_next_step(self, form, **kwargs):
         """
         In the validation step, ask the device to generate a challenge.
@@ -486,8 +535,9 @@ class SetupView(IdempotentSessionWizardView):
         if method.code == 'generator':
             form = [form for form in form_list if isinstance(form, TOTPDeviceForm)][0]
             device = form.save()
-        # PhoneNumberForm / YubiKeyDeviceForm / EmailForm
-        elif method.code in ('call', 'sms', 'yubikey', 'email'):
+
+        # PhoneNumberForm / YubiKeyDeviceForm / EmailForm / WebauthnDeviceValidationForm
+        elif method.code in ('call', 'sms', 'yubikey', 'email', 'webauthn'):
             device = self.get_device()
             device.save()
 
@@ -495,19 +545,25 @@ class SetupView(IdempotentSessionWizardView):
             raise NotImplementedError("Unknown method '%s'" % method.code)
 
         django_otp.login(self.request, device)
-        return redirect(self.success_url)
+        return redirect(self.get_success_url())
 
     def get_form_kwargs(self, step=None):
+        if step is None:
+            return {}
+
+        form_class = self.get_form_list()[step]
+        form_params = signature(form_class).parameters
+
         kwargs = {}
-        if step == 'generator':
-            kwargs.update({
-                'key': self.get_key(step),
-                'user': self.request.user,
-            })
-        if step in ('validation', 'yubikey'):
-            kwargs.update({
-                'device': self.get_device()
-            })
+        if 'key' in form_params:
+            kwargs['key'] = self.get_key(step)
+        if 'user' in form_params:
+            kwargs['user'] = self.request.user
+        if 'device' in form_params:
+            kwargs['device'] = self.get_device()
+        if 'request' in form_params:
+            kwargs['request'] = self.request
+
         metadata = self.get_form_metadata(step)
         if metadata:
             kwargs.update({
@@ -542,10 +598,18 @@ class SetupView(IdempotentSessionWizardView):
             key = self.get_key('generator')
             rawkey = unhexlify(key.encode('ascii'))
             b32key = b32encode(rawkey).decode('utf-8')
+            issuer = get_current_site(self.request).name
+            username = self.request.user.get_username()
+            otpauth_url = get_otpauth_url(username, b32key, issuer)
             self.request.session[self.session_key_name] = b32key
             context.update({
+                # used in default template
+                'otpauth_url': otpauth_url,
                 'QR_URL': reverse(self.qrcode_url),
                 'secret_key': b32key,
+                # available for custom templates
+                'issuer': issuer,
+                'totp_digits': totp_digits(),
             })
         elif self.steps.current == 'validation':
             context['device'] = self.get_device()
@@ -607,6 +671,11 @@ class SetupCompleteView(TemplateView):
     """
     template_name = 'two_factor/core/setup_complete.html'
 
+    def get(self, request, *args, **kwargs):
+        if request.session.get('next'):
+            return redirect(request.session.get('next'))
+        return super().get(request, *args, **kwargs)
+
     def get_context_data(self):
         return {
             'phone_methods': get_available_phone_methods(),
